#!/usr/common/bin/wish8.0 -f

#
# James Davis - Sept 2000
# 
# Tool for helping control excessive debugging output
#

# Build the main window
proc db_buildWindow {} {

    # rules list box
    frame .lf
    listbox .lf.list \
	-yscrollcommand ".lf.scroll set" -selectmode multiple \
	-exportselection false
    scrollbar .lf.scroll -command ".lf.list yview"
    pack .lf.list .lf.scroll -side left -fill both

    bind .lf.list <ButtonRelease-1> {
	set recentClick [%W index @%x,%y]
	if {$recentClick >= 0} {
	    set rule_active($list_to_rule($recentClick)) [.lf.list selection include $recentClick]
	}
    }
    
  #  bind . <Enter> db_fixlistselection

    frame .b
    button .b.quick -text Quick -command {db_clipbutton}
    button .b.new -text New -command {db_newrule}
    button .b.edit -text Edit -command {db_editbutton}
    button .b.delete -text Delete -command {db_deletebutton}
    button .b.help -text Help -command db_helpbutton
    button .b.save -text Save -command db_save
    button .b.quit -text Quit -command {db_save; exit}

    # Create image if it exists
    global env
    if {[info exists env(DBUG_DIR)] && [file exists $env(DBUG_DIR)/logo.ppm]} {
	set im [image create photo -file $env(DBUG_DIR)/logo.ppm]
	button .b.logo -image $im
    }
    packchildren .b -fill x  

    # stdout textbox
  #  frame .tf
  #  text .tf.text \
\#	-yscrollcommand ".tf.scroll set"
  #  scrollbar .tf.scroll -command ".tf.text yview"
  #  pack .tf.scroll .tf.text -side right -fill both

    # pack everythign
    pack  .b .lf -side left -fill both

}

# Bring u pthe help text box
proc db_helpbutton {} {
    db_globals
    if [window_Activate .help] return

    set h [toplevel .help]
    wm title $h Help

    text $h.text -yscrollcommand "$h.scroll set" 
    scrollbar $h.scroll -command "$h.text yview" 

    packchildren $h -side left -fill y

    $h.text insert end {
-----------------------------------------------------------------------------
| dbug - James Davis - Sep 2000
-----------------------------------------------------------------------------

'dbug' is a tool that allows you to manage the output of applications
that are perhaps a bit _too_ verbose on stdout. 'dbug' maintains a list
of rules which are used to selectively allow/hide output lines. Each rule
has a regular expression, and an optional title. 'dbug' is invoked as a pipe
on stdout of your application, and suppress output lines that match 
active rules. Click on rules in the rule list box to toggle their active
status.

Quick  - Quickly create a new rule that filters out lines with text
            matching the current X selection.
New    - Create a new rule, using the full dialog to specify regexp and title.
Edit   - Edit the rule most recently clicked in the rule list box.
Delete - Delete the rule most recently clicked in the rule list box.
Help   - Bring up this message.
Save   - Save the existing set of rules to a .dbug file in the current
            directory. This file is read on startup.
Quit   - First save rules, then exit.
    }
}

# Try to make a new rule based on current X selection
proc db_clipbutton {} {
    db_globals

    set sel [ selection get]
    if {$sel != ""} {
	# Make a new rule
	set num [db_addrule]
	# Make it active
	set rule_active($num) 1
	# Set the regexp
	set rule_regexp($num) $sel
	# Show the rule
	db_showrule $num
    }
}

# Save the rules to a .dbug file for later loading
proc db_save {} {
    db_globals

    set numRules [array size rule_regexp]



    # Open the file to write to
    set fid ""
    catch {set fid [open .dbug w]}
    if {$fid==""} {
	puts "Can not write .dbug file"
	return
    }

    # Write some prep stuff
    puts $fid "db_globals"

    # Loop over rules
    for {set i 1} { $i <= $numRules} {incr i} {
	# If this rule is visible then save it
	if {$rule_visible($i)} {
	    puts $fid {set num [db_addrule]}
	    puts $fid "set rule_title(\$num) \"$rule_title($i)\""
	    puts $fid "set rule_regexp(\$num) \"$rule_regexp($i)\""
	    puts $fid "set rule_active(\$num) $rule_active($i)"
	    puts $fid "db_showrule \$num"
	    puts $fid "\#"
	}
    }

    # Close file
    close $fid

}

proc db_deletebutton {} {
    db_globals
    if {$recentClick== -1} return;
    if {$recentClick==""} return;

    set rule $list_to_rule($recentClick)
    db_deleterule $rule
}

# Delete a single rule
proc db_deleterule {num} {
    db_globals

    #puts "ltr srt [array size list_to_rule] lbs [.lf.list index end]"
   # db_dumplisttorule
    # Basically lazy, don't really delete, just make it
    # not visible, and pretend its gone
    if {$rule_visible($num)==1} {
	# Remove from the listbox
	.lf.list delete $rule_to_list($num)

	# Remove from list_to_rule and rule_to_list mapping
	set sz [array size list_to_rule]
	set sz [expr $sz - 1]
	set li $rule_to_list($num)
	for {set i $li} {$i < $sz} { incr i} {
	    #puts "setting ltr($i) $list_to_rule([expr $i + 1])"
	    set list_to_rule($i) $list_to_rule([expr $i + 1])
	    set rl $list_to_rule($i)
	    #puts "setting rtl($rl) $i"
	    set rule_to_list($rl) $i
	}
	unset list_to_rule($sz)

	# Reset the rule_to_list pointer
	set rule_to_list($num) "-1"

	# Make the rule inactive
	set rule_active($num) 0

	# Make the rule invisible
	set rule_visible($num) 0
    }
    #puts "ltr end [array size list_to_rule] lbs [.lf.list index end]"
    #db_dumplisttorule

}

#debuging routine
proc db_dumplisttorule {} {
    db_globals

    puts ""
    set sz [array size list_to_rule]
    for {set i 0} {$i < $sz} {incr i} {
	puts "$i -> rule $list_to_rule($i) $rule_regexp($list_to_rule($i)) -> $rule_to_list($list_to_rule($i))"
    }
    puts ""
}

# Runs on startup to load a previously saved .dbug file
proc db_load {} {
    if [file readable .dbug] {
	source .dbug
    }
}

# Someone hit the Edit button
proc db_editbutton {} {
    db_globals
    
    if {$recentClick >= 0} {
	db_editrule $list_to_rule($recentClick)
    }
}

# Bring up the dialog for editing a rule
proc db_editrule {num} {
    db_globals
    if [window_Activate .edit] return

    set e [toplevel .edit]
    wm title $e Edit

    label $e.tlab -text "Title"
    entry $e.title -textvariable rule_title($num)
    label $e.rlab -text "Regular Expression"
    entry $e.regexp -textvariable rule_regexp($num)

    # The ok button, same as binding below
    button $e.ok -text Ok -command "db_editOKbutton $num"

    # Bind the return key to hit ok
    bind $e <Return> "db_editOKbutton $num"
  
    pack $e.tlab $e.title $e.rlab $e.regexp -fill x
    pack $e.ok
}

proc db_editOKbutton {num} {
    db_globals
    
    db_showrule $num
    if {$rule_regexp($num)==""} {
	db_deleterule $num
    } 

    destroy .edit
}

# Hit the New button
proc db_newrule {} {
    db_globals
    
    # Make a new rule
    set num [db_addrule]
    # Make it active
    set rule_active($num) 1
    # Edit the rule
    set res [db_editrule $num]

 
  
}

# Show or update a rule in the gui listbox
proc db_showrule {num} {
    db_globals
   
    # Set the desired text
    set text $rule_regexp($num)
    if {$rule_title($num)!=""} {
	set text $rule_title($num)
    }

    # Check if we are already visisble
    if {$rule_visible($num)==1} {
	# change the text in the list box
	.lf.list delete $rule_to_list($num)
	.lf.list insert $rule_to_list($num) $text
    } else {
	
	# set the rule visible
	set rule_visible($num) 1
	
	# make an inverse pointer
	set sz [array size list_to_rule]
	set list_to_rule($sz) $num

	# set the forward link
	set rule_to_list($num) $sz

	# add to the list box
	.lf.list insert $sz $text
    }

    # Try to fix up the active status
    set ind $rule_to_list($num)
    if {$rule_active($num)==1} {
	# Set selected
	.lf.list selection set $ind
    } else {
	# Set not selected
	.lf.list selection clear $ind
    }
}

# Step through each list element and set the selection correctly
# now obsolete since I figured out how to set the list not to 
# be the X selection, used to bind this to entering the window
proc db_fixlistselection {} {
    db_globals
    set sz [array size list_to_rule]
    for {set i 0} {$i < $sz} {incr i} {
	set num $list_to_rule($i)
	
	# Try to fix up the active status
	set ind $rule_to_list($num)
	if {$rule_active($num)==1} {
	    # Set selected
	    .lf.list selection set $ind
	} else {
	    # Set not selected
	    .lf.list selection clear $ind
	}
    }
}

# Init stuff that needs to run
proc db_init {} {
    db_globals

    # Build the main window
    db_buildWindow

    # Load any previously saved rules
    db_load
  
    # Set a handler for things on stdin
    fileevent stdin readable db_processLine
    
    # Set stdin to be nonblocking
    fconfigure stdin -blocking 0

}

#obsolete
proc db_checkLines {} {
    # Set time slice size
    set slice 100

    # Arrange to process for limited time
    global checkStop
    set checkStop 0
    after $slice {set checkStop 1}

    # Process as many lines as we can in the time
    puts START
    while {!$checkStop && ![eof stdin]} {
	# Process lines
	db_processLine

	# Critical, to allow checkStop to be set use update
	update
    }
    puts END

    # Arrange to get another line processing time slice
    after $slice db_checkLines
}

# Process a single line of input
proc db_processLine {} { 
    db_globals
    
   # puts "START"

    # get line
    set newline ""
    set count [gets stdin line]
    # Check if we got a whole line, if not get a partial line
    if {$count <0} {
	set line [read stdin]
	set newline "-nonewline"
    }
    set numrule [array size rule_regexp]
    set valid 1

    # loop over rules
    for {set i 1} {$i <= $numrule} {incr i} {
	if {$rule_active($i) && [regexp $rule_regexp($i) $line]} {
	    set valid 0
	}
    }

    # if valid then puts to stdout
    if {$valid==1} {
	# Figure out whether to include a newline
	if {$newline==""} {
	    puts $line
	} else {
	    puts -nonewline "$line"
	    flush stdout
	}
    }

    # check for create a new rule
  #  puts "END"

}

# Add a rule to our data structures
proc db_addrule {} {
    db_globals

    set sz [array size rule_regexp]
    incr sz
    set rule_title($sz) ""
    set rule_regexp($sz) ""
    set rule_active($sz) 0
    set rule_visible($sz) 0
    set rule_observations($sz) 0
    set rule_to_list($sz) 0

    return $sz
}

# Set all the globals
proc db_globals {} {
    uplevel {
	global rule_title
	global rule_regexp
	global rule_active
	global rule_visible
	global rule_observations
	global list_to_rule
	global rule_to_list
	globalinit recentClick
    }
}

# obsolete
proc db_update {} {
    update
    after 100 db_update
}

#####
# Util routines from other places
#####

proc window_Activate {window} {
    if {[winfo exists $window]} {
        # bring it to foreground
        # don't move the window around 
        # (at least not by more than required by grid)
        set geom [wm geometry $window]
        #puts $geom
        wm withdraw $window
        wm deiconify $window
        wm geometry $window $geom
        return 1
    } else {
        return 0
    }
}

proc packchildren {parent args} {
    foreach child [winfo children $parent] {
        eval pack $child $args
    }
}

proc globalinit {var {val ""}} {
    global $var
    uplevel "global $var"

    if {![info exists $var]} {
        set $var $val
    }
}



# Main stuff to run
db_init
